#include "math.h"
#include "drawbot.h" 
#include "AT91SAM7X256.h"
#include "types.h"
#include "timer.h"
#include "timer_internal.h"
#include "config.h"
#include "rtos.h"
#include "line.h"
  
#define NUMBER_OF_STEPS_TO_MAX_SPEED 7
#define INITIAL_VELOCITY 25
#define VELOCITY_INCREMENT 50
#define TIMING_CONSTANT 30000

int timeSync;

/**
 *
 **/
void Swap(int* x, int *y)
{
    int z = *x;
    *x = *y;
    *y = z;
} //END swap

/**
 *
 **/
void MoveX(int dir)
{ 
    DigitalOut_SetValue( X_AXIS_DIR_OUTPUT, (dir < 0 ? X_BACK : X_FORWARD));
    DigitalOut_SetValue( X_AXIS_STEP_OUTPUT, LOW);
    DigitalOut_SetValue( X_AXIS_STEP_OUTPUT, HIGH);
    AppLed_SetState( 1, dir);  
    AppLed_SetState( 0, HIGH); 
    AppLed_SetState( 0, LOW);
} //END moveX

/**
 *
 **/
void MoveY(int dir)
{
    
    DigitalOut_SetValue( Y_AXIS_DIR_OUTPUT, (dir < 0 ? Y_BACK : Y_FORWARD));
    DigitalOut_SetValue( Y_AXIS_STEP_OUTPUT, LOW);
    DigitalOut_SetValue( Y_AXIS_STEP_OUTPUT, HIGH);
    AppLed_SetState(3, dir); 
    AppLed_SetState(2, HIGH);
    AppLed_SetState(2, LOW);  
} //END moveY

/**
 *
 **/
void MoveZ(int dir) 
{
 
    DigitalOut_SetValue( Z_AXIS_DIR_OUTPUT, dir);
    DigitalOut_SetValue( Z_AXIS_STEP_OUTPUT, HIGH);
    DigitalOut_SetValue( Z_AXIS_STEP_OUTPUT, LOW);
	
} //END moveZ

/**
 *
 **/
int GetTicks(void)
{
     return AT91C_BASE_TC0->TC_CV;
} //END getTicks

/**
 *
 **/
int StepToDelay(int step)
{
    int velocity = step * VELOCITY_INCREMENT + INITIAL_VELOCITY;
    int delay = TIMING_CONSTANT / velocity;
    return delay;
} //END steToDelay


/**
 *
 **/
void MoveTo(int x0, int y0, int x1, int y1) {

 	MovePen(UP); 
	DrawLine(x0,y0,x1,y1); 
	MovePen(DOWN);
	
} //END moveLine

/**
 *
 */
void DrawLine(int x0, int y0, int x1, int y1)
{
    char steep;
    int deltax, deltay, error, i, mid, speed, xstep, ystep;

    xstep = ((x0 < x1) ? 1 : -1);
    ystep = ((y0 < y1) ? 1 : -1);
    
    steep = (abs(y1 - y0) > abs(x1 - x0));
    if (steep)
    {
        Swap(&x0, &y0);
        Swap(&x1, &y1);
    }
    
    deltax = abs(x1 - x0);
    deltay = abs(y1 - y0);
    error = 0;

    mid = deltax / 2;
    for (i = 0; i < deltax; i++)
    {
	vTaskDelay(MOVE_DELAY);
       
        if (steep) MoveY(ystep);
        else MoveX(xstep);
       
        error += deltay;
        if (2 * error >= deltax)
        {
            if (steep) MoveX(xstep);
            else MoveY(ystep);

            error -= deltax;
        }
    }
} //END drawLine

/**
 *
 **/
void DrawPoint(int x, int y)
{
  static char skip = 1;
  static int lastX = 0;
  static int lastY = 0;
  if (skip)
  {
     skip = 0;
     lastX = x;
     lastY = y;
     return;
  }
  DrawLine(lastX, lastY, x, y);
  lastX = x;
  lastY = y;
} //END drawPoint

/**
 *
 **/
void MovePen(int dir) {
 
 int i = 0;

 for( i = 0; i < 40; i++ ) {  
    MoveZ(dir); 	
    vTaskDelay(MOVE_DELAY); 
 }
 
} //END movePen

/**
  *
  **/
int HandleChar(char c)
{
    static int state = 0;   // 0 = parseing x
    static int parX = 0;
    static int parY = 0;
    static int lastX = 0;
    static int lastY = 0;
    static int wasEnd = 1;


    // if char s sleep 10 seconds  
    if (c == 's') {
       vTaskDelay(WAIT_DELAY);
       return 1;
      } 
    // if char q quit drawing
    if (c == 'q') 
       return 0;

    //state 0 = x  coord , state 1 = y  coord
    switch(state)
    {
    // X
    case 0:
        if (c >= '0' && c <= '9')
        {
	    // shift X param 10x
            parX *= 10;
	    // subtract char value from char and add to parm X
            parX += (c - '0');
        }

        else if (c == ',')
            // start parsing y 
            state = 1;
        else if (c == '\n') 
            // END of line
            wasEnd = 1;
	    else {
              return 0;
	    }
    break;
    // Y
    case 1:
        if (c >= '0' && c <= '9')
        {
            parY *= 10;
            parY += (c - '0');
        }
        else if (c == '\n')
        {
            if (wasEnd)
                MoveTo(lastX, lastY, parX, parY);
            else
                DrawLine(lastX, lastY, parX, parY);
	    
            wasEnd = 0;
            lastX = parX;
            lastY = parY;
            parX = 0;
            parY = 0;
	    state = 0; 
	    
        } else {
            // ERROR
            return 0;
	}

        break;
    }
    return 1;
}//END handleChar  

/**
  *
  **/ 
int ParseInput(char *buf, int len)
{  
    int i = 0; 
    for(i = 0; i < len  ; i++)
    {
	    if (HandleChar(buf[i]) == 0) {
              return 0;
	    }
    }
    return 1;
} //END parseInput

